# import unittest
# import common.variables as variables
# from server import process_client_msg

# # корректные данные
# ACTION_GOOD = variables.PRESENCE
# USER_GOOD = "Sergey"
# PORT = variables.DEFAULT_PORT
# TIME = 1.1
# RESPONSE_GOOD = {variables.RESPONSE: 200}

# # некорректные данные
# USER_BAD = "Petya"
# RESPONSE_BAD = {variables.RESPONSE: 400, variables.ERROR: "УПС, ошибочка..."}


# class TestServer(unittest.TestCase):
#     def test_process_client_msg_correct(self):
#         client_message = {
#             variables.ACTION: ACTION_GOOD,
#             variables.TIME: TIME,
#             variables.PORT: PORT,
#             variables.USER: {variables.ACCOUNT_NAME: USER_GOOD},
#         }
#         self.assertEqual(process_client_msg(client_message), RESPONSE_GOOD)

#     def test_process_client_msg_not_user(self):
#         client_message = {
#             variables.ACTION: ACTION_GOOD,
#             variables.TIME: TIME,
#             variables.PORT: PORT,
#             variables.USER: {variables.ACCOUNT_NAME: USER_BAD},
#         }
#         self.assertEqual(process_client_msg(client_message), RESPONSE_BAD)

#     def test_process_client_msg_not_port(self):
#         client_message = {
#             variables.ACTION: ACTION_GOOD,
#             variables.TIME: TIME,
#             variables.USER: {variables.ACCOUNT_NAME: USER_BAD},
#         }
#         self.assertEqual(process_client_msg(client_message), RESPONSE_BAD)

#     def test_process_client_msg_port(self):
#         client_message = {
#             variables.ACTION: ACTION_GOOD,
#             variables.TIME: TIME,
#             variables.PORT: 8080,
#             variables.USER: {variables.ACCOUNT_NAME: USER_BAD},
#         }
#         self.assertEqual(process_client_msg(client_message), RESPONSE_BAD)


# if __name__ == "__main__":
#     unittest.main()



# from common.errors_user import ReqFieldMissingError, ServerError
# import unittest
# from common.variables import *
# from client import create_presence, process_response_ans
# import sys

# sys.path.append("../")


# # Класс с тестами
# class TestClass(unittest.TestCase):
#     # тест коректного запроса
#     def test_def_presense(self):
#         test = create_presence("Guest")
#         # время необходимо приравнять принудительно иначе тест никогда не будет пройден
#         test[TIME] = 1.1
#         self.assertEqual(
#             test, {ACTION: PRESENCE, TIME: 1.1, USER: {ACCOUNT_NAME: "Guest"}}
#         )

#     # тест корректтного разбора ответа 200
#     def test_200_ans(self):
#         self.assertEqual(process_response_ans({RESPONSE: 200}), "200 : OK")

#     # тест корректного разбора 400
#     def test_400_ans(self):
#         self.assertRaises(
#             ServerError, process_response_ans, {RESPONSE: 400, ERROR: "Bad Request"}
#         )

#     # тест исключения без поля RESPONSE
#     def test_no_response(self):
#         self.assertRaises(
#             ReqFieldMissingError, process_response_ans, {ERROR: "Bad Request"}
#         )


# if __name__ == "__main__":
#     unittest.main()



from PyQt5.QtWidgets import QDialog, QLabel, QComboBox, QPushButton, QApplication
from PyQt5.QtCore import Qt
from PyQt5.QtGui import QStandardItemModel, QStandardItem

from ..core.core import MessageProcessor
from ..db_server import storage


class DelUserDialog(QDialog):
    """
    Класс - диалог выбора контакта для удаления.
    """

    def __init__(self, db: storage, server: MessageProcessor):
        super().__init__()
        self.db = db
        self.server = server

        self.setFixedSize(350, 120)
        self.setWindowTitle("Удаление пользователя")
        self.setAttribute(Qt.WA_DeleteOnClose)
        self.setModal(True)

        self.selector_label = QLabel("Выберите пользователя для удаления:", self)
        self.selector_label.setFixedSize(200, 20)
        self.selector_label.move(10, 0)

        self.selector = QComboBox(self)
        self.selector.setFixedSize(200, 20)
        self.selector.move(10, 30)

        self.btn_ok = QPushButton("Удалить", self)
        self.btn_ok.setFixedSize(100, 30)
        self.btn_ok.move(230, 20)
        self.btn_ok.clicked.connect(self.remove_user)

        self.btn_cancel = QPushButton("Отмена", self)
        self.btn_cancel.setFixedSize(100, 30)
        self.btn_cancel.move(230, 60)
        self.btn_cancel.clicked.connect(self.close)

        self.all_users_fill()

    def all_users_fill(self):
        """Метод заполняющий список пользователей."""
        self.selector.addItems([item[0] for item in self.db.users_list()])

    def remove_user(self):
        """Метод - обработчик удаления пользователя."""
        username = self.selector.currentText()
        if self.db.check_user_existing(username):
            self.db.remove_user(username)

            if username in self.server.names:
                sock = self.server.names[username]
                del self.server.names[username]
                self.server.remove_client(sock)
            # Рассылаем клиентам сообщение о необходимости обновить справочники
            self.server.service_update_lists()
            self.close()
        else:
            self.messages.critical(self, "Ошибка", "Не удалось удалить пользователя.")
            return



import subprocess

if __name__ == "__main__":
    process = []
    while True:
        action = input(
            'Выберите действие: q - выход , s - запустить сервер и клиенты, x - закрыть все окна:')
        if action == 'q':
            break
        elif action == 's':
            clients_count = int(
                input('Введите количество тестовых клиентов для запуска: '))
            # Запускаем сервер!
            process.append(subprocess.Popen('python server_run.py',
                        creationflags=subprocess.CREATE_NEW_CONSOLE))
            # Запускаем клиентов:
            for i in range(clients_count):
                process.append(subprocess.Popen(
                    f'python client.py -n test{i + 1}', creationflags=subprocess.CREATE_NEW_CONSOLE))
        elif action == 'x':
            while process:
                process.pop().kill()


# Логгерский конфиг для сервера

import sys
import os

import logging.handlers
import logging

from ..core.variables import LOGGING_LEVEL


# формировщик логов (formatter):
server_formatter = logging.Formatter("%(asctime)s %(levelname)s %(filename)s %(message)s")

# вывод в консоль
steam = logging.StreamHandler(sys.stderr)
steam.setFormatter(server_formatter)
steam.setLevel(logging.INFO)

# вывод в файл
path = os.path.dirname(os.path.abspath(__file__))
path = os.path.join(path, "server.log")  # Подготовка имени файла для логирования
log_file = logging.handlers.TimedRotatingFileHandler(path, encoding="utf8", interval=1, when="D")
log_file.setFormatter(server_formatter)

# создаём регистратор и настраиваем его
logger = logging.getLogger("server")
logger.addHandler(steam)
logger.addHandler(log_file)
logger.setLevel(LOGGING_LEVEL)


def log(func_to_log):
    """
    Декоратор, выполняющий логирование вызовов функций.
    Сохраняет события типа debug, содержащие
    информацию о имени вызываемой функиции, параметры с которыми
    вызывается функция, и модуль, вызывающий функцию.
    """

    def log_saver(*args, **kwargs):
        logger.debug(
            f"Была вызвана функция {func_to_log.__name__} c параметрами {args} , {kwargs}. "
            f"Вызов из модуля {func_to_log.__module__}"
        )
        ret = func_to_log(*args, **kwargs)
        return ret

    return log_saver


# отладка
if __name__ == "__main__":
    logger.critical("Test critical event")
    logger.error("Test error ivent")
    logger.debug("Test debug ivent")
    logger.info("Test info ivent")



# from common.errors_user import NonDictInputError
# import unittest
# from common.variables import *
# from common.utils import *
# import sys

# sys.path.append("../")


# # Тестовый класс для тестирования отпраки и получения, при создании требует словарь, который будет прогонятся
# # через тестовую функцию
# class TestSocket:
#     def __init__(self, test_dict):
#         self.testdict = test_dict

#     # тестовая функция отправки, корретно  кодирует сообщение, так-же сохраняет что должно было отправлено в сокет.
#     def send(self, message_to_send):
#         json_test_message = json.dumps(self.testdict)
#         self.encoded_message = json_test_message.encode(ENCODING)
#         self.receved_message = message_to_send

#     def recv(self, max_len):
#         json_test_message = json.dumps(self.testdict)
#         return json_test_message.encode(ENCODING)


# # Тестовый класс, собственно выполняющий тестирование.
# class Tests(unittest.TestCase):
#     test_dict_send = {
#         ACTION: PRESENCE,
#         TIME: 111111.111111,
#         USER: {ACCOUNT_NAME: "test_test"},
#     }
#     test_dict_recv_ok = {RESPONSE: 200}
#     test_dict_recv_err = {RESPONSE: 400, ERROR: "Bad Request"}

#     # тестируем корректность работы фукции отправки,создадим тестовый сокет и проверим корректность отправки словаря
#     def test_send_message(self):
#         # экземпляр тестового словаря, хранит собственно тестовый словарь
#         test_socket = TestSocket(self.test_dict_send)
#         # вызов тестируемой функции, результаты будут сохранены в тестовом сокете
#         send_message(test_socket, self.test_dict_send)
#         # проверка корретности кодирования словаря. сравниваем результат довренного кодирования и результат от тестируемой функции
#         self.assertEqual(test_socket.encoded_message, test_socket.receved_message)
#         # дополнительно, проверим генерацию исключения, при не словаре на входе.
#         self.assertRaises(NonDictInputError, send_message, test_socket, 1111)

#     # тест функции приёма сообщения
#     def test_get_message(self):
#         test_sock_ok = TestSocket(self.test_dict_recv_ok)
#         test_sock_err = TestSocket(self.test_dict_recv_err)
#         # тест корректной расшифровки корректного словаря
#         self.assertEqual(get_message(test_sock_ok), self.test_dict_recv_ok)
#         # тест корректной расшифровки ошибочного словаря
#         self.assertEqual(get_message(test_sock_err), self.test_dict_recv_err)


# if __name__ == "__main__":
#     unittest.main()
